# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawLocationProfilesClient
import typing
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.location_booking_profile import LocationBookingProfile
from ...types.list_location_booking_profiles_response import ListLocationBookingProfilesResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawLocationProfilesClient
from ...core.pagination import AsyncPager


class LocationProfilesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawLocationProfilesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawLocationProfilesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawLocationProfilesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[LocationBookingProfile]:
        """
        Lists location booking profiles of a seller.

        Parameters
        ----------
        limit : typing.Optional[int]
            The maximum number of results to return in a paged response.

        cursor : typing.Optional[str]
            The pagination cursor from the preceding response to return the next page of the results. Do not set this when retrieving the first page of the results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[LocationBookingProfile]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.bookings.location_profiles.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/bookings/location-booking-profiles",
            method="GET",
            params={
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListLocationBookingProfilesResponse,
                    construct_type(
                        type_=ListLocationBookingProfilesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.location_booking_profiles
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncLocationProfilesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawLocationProfilesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawLocationProfilesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawLocationProfilesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[LocationBookingProfile]:
        """
        Lists location booking profiles of a seller.

        Parameters
        ----------
        limit : typing.Optional[int]
            The maximum number of results to return in a paged response.

        cursor : typing.Optional[str]
            The pagination cursor from the preceding response to return the next page of the results. Do not set this when retrieving the first page of the results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[LocationBookingProfile]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.bookings.location_profiles.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/bookings/location-booking-profiles",
            method="GET",
            params={
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListLocationBookingProfilesResponse,
                    construct_type(
                        type_=ListLocationBookingProfilesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.location_booking_profiles
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .destination_type import DestinationType
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Destination(UncheckedBaseModel):
    """
    Information about the destination against which the payout was made.
    """

    type: typing.Optional[DestinationType] = pydantic.Field(default=None)
    """
    Type of the destination such as a bank account or debit card.
    See [DestinationType](#type-destinationtype) for possible values
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Square issued unique ID (also known as the instrument ID) associated with this destination.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class EventDataParams(typing_extensions.TypedDict):
    type: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the affected object’s type.
    """

    id: typing_extensions.NotRequired[str]
    """
    The ID of the affected object.
    """

    deleted: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    This is true if the affected object has been deleted; otherwise, it's absent.
    """

    object: typing_extensions.NotRequired[typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]]
    """
    An object containing fields and values relevant to the event. It is absent if the affected object has been deleted.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .money import MoneyParams
from ..types.gift_card_activity_adjust_decrement_reason import GiftCardActivityAdjustDecrementReason


class GiftCardActivityAdjustDecrementParams(typing_extensions.TypedDict):
    """
    Represents details about an `ADJUST_DECREMENT` [gift card activity type](entity:GiftCardActivityType).
    """

    amount_money: MoneyParams
    """
    The amount deducted from the gift card balance. This value is a positive integer.
    """

    reason: GiftCardActivityAdjustDecrementReason
    """
    The reason the gift card balance was adjusted.
    See [Reason](#type-reason) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing

LocationCapability = typing.Union[
    typing.Literal["CREDIT_CARD_PROCESSING", "AUTOMATIC_TRANSFERS", "UNLINKED_REFUNDS"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class BulkSwapPlanResponseParams(typing_extensions.TypedDict):
    """
    Defines output parameters in a response of the
    [BulkSwapPlan](api-endpoint:Subscriptions-BulkSwapPlan) endpoint.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors encountered during the request.
    """

    affected_subscriptions: typing_extensions.NotRequired[int]
    """
    The number of affected subscriptions.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.payout_status import PayoutStatus
from .money import MoneyParams
from .destination import DestinationParams
from ..types.payout_type import PayoutType
import typing
from .payout_fee import PayoutFeeParams


class PayoutParams(typing_extensions.TypedDict):
    """
    An accounting of the amount owed the seller and record of the actual transfer to their
    external bank account or to the Square balance.
    """

    id: str
    """
    A unique ID for the payout.
    """

    status: typing_extensions.NotRequired[PayoutStatus]
    """
    Indicates the payout status.
    See [PayoutStatus](#type-payoutstatus) for possible values
    """

    location_id: str
    """
    The ID of the location associated with the payout.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp of when the payout was created and submitted for deposit to the seller's banking destination, in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp of when the payout was last updated, in RFC 3339 format.
    """

    amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money involved in the payout. A positive amount indicates a deposit, and a negative amount indicates a withdrawal. This amount is never zero.
    """

    destination: typing_extensions.NotRequired[DestinationParams]
    """
    Information about the banking destination (such as a bank account, Square checking account, or debit card)
    against which the payout was made.
    """

    version: typing_extensions.NotRequired[int]
    """
    The version number, which is incremented each time an update is made to this payout record.
    The version number helps developers receive event notifications or feeds out of order.
    """

    type: typing_extensions.NotRequired[PayoutType]
    """
    Indicates the payout type.
    See [PayoutType](#type-payouttype) for possible values
    """

    payout_fee: typing_extensions.NotRequired[typing.Optional[typing.Sequence[PayoutFeeParams]]]
    """
    A list of transfer fees and any taxes on the fees assessed by Square for this payout.
    """

    arrival_date: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The calendar date, in ISO 8601 format (YYYY-MM-DD), when the payout is due to arrive in the seller’s banking destination.
    """

    end_to_end_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID for each `Payout` object that might also appear on the seller’s bank statement. You can use this ID to automate the process of reconciling each payout with the corresponding line item on the bank statement.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .vendor import VendorParams


class GetVendorResponseParams(typing_extensions.TypedDict):
    """
    Represents an output from a call to [RetrieveVendor](api-endpoint:Vendors-RetrieveVendor).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors encountered when the request fails.
    """

    vendor: typing_extensions.NotRequired[VendorParams]
    """
    The successfully retrieved [Vendor](entity:Vendor) object.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class RetrieveTokenStatusResponse(UncheckedBaseModel):
    """
    Defines the fields that are included in the response body of
    a request to the `RetrieveTokenStatus` endpoint.
    """

    scopes: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of scopes associated with an access token.
    """

    expires_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The date and time when the `access_token` expires, in RFC 3339 format. Empty if the token never expires.
    """

    client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-issued application ID associated with the access token. This is the same application ID used to obtain the token.
    """

    merchant_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the authorizing merchant's business.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
     Any errors that occurred during the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .update_team_member_response import UpdateTeamMemberResponse
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BatchUpdateTeamMembersResponse(UncheckedBaseModel):
    """
    Represents a response from a bulk update request containing the updated `TeamMember` objects or error messages.
    """

    team_members: typing.Optional[typing.Dict[str, UpdateTeamMemberResponse]] = pydantic.Field(default=None)
    """
    The successfully updated `TeamMember` objects. Each key is the `team_member_id` that maps to the `UpdateTeamMemberRequest`.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    The errors that occurred during the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class SiteParams(typing_extensions.TypedDict):
    """
    Represents a Square Online site, which is an online store for a Square seller.
    """

    id: typing_extensions.NotRequired[str]
    """
    The Square-assigned ID of the site.
    """

    site_title: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The title of the site.
    """

    domain: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The domain of the site (without the protocol). For example, `mysite1.square.site`.
    """

    is_published: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the site is published.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp of when the site was created, in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp of when the site was last updated, in RFC 3339 format.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

ErrorCategory = typing.Union[
    typing.Literal[
        "API_ERROR",
        "AUTHENTICATION_ERROR",
        "INVALID_REQUEST_ERROR",
        "RATE_LIMIT_ERROR",
        "PAYMENT_METHOD_ERROR",
        "REFUND_ERROR",
        "MERCHANT_SUBSCRIPTION_ERROR",
        "EXTERNAL_VENDOR_ERROR",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CatalogQueryItemsForItemOptionsParams(typing_extensions.TypedDict):
    """
    The query filter to return the items containing the specified item option IDs.
    """

    item_option_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    A set of `CatalogItemOption` IDs to be used to find associated
    `CatalogItem`s. All Items that contain all of the given Item Options (in any order)
    will be returned.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .shift_filter import ShiftFilterParams
from .shift_sort import ShiftSortParams


class ShiftQueryParams(typing_extensions.TypedDict):
    """
    The parameters of a `Shift` search query, which includes filter and sort options.

    Deprecated at Square API version 2025-05-21. See the [migration notes](https://developer.squareup.com/docs/labor-api/what-it-does#migration-notes).
    """

    filter: typing_extensions.NotRequired[ShiftFilterParams]
    """
    Query filter options.
    """

    sort: typing_extensions.NotRequired[ShiftSortParams]
    """
    Sort order details.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class SubscriptionSourceParams(typing_extensions.TypedDict):
    """
    The origination details of the subscription.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name used to identify the place (physical or digital) that
    a subscription originates. If unset, the name defaults to the name
    of the application that created the subscription.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class AddGroupToCustomerResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the [AddGroupToCustomer](api-endpoint:Customers-AddGroupToCustomer) endpoint.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.get_wage_setting_response import GetWageSettingResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.wage_setting import WageSettingParams
from ...types.update_wage_setting_response import UpdateWageSettingResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawWageSettingClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, team_member_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetWageSettingResponse]:
        """
        Retrieves a `WageSetting` object for a team member specified
        by `TeamMember.id`. For more information, see
        [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#retrievewagesetting).

        Square recommends using [RetrieveTeamMember](api-endpoint:Team-RetrieveTeamMember) or [SearchTeamMembers](api-endpoint:Team-SearchTeamMembers)
        to get this information directly from the `TeamMember.wage_setting` field.

        Parameters
        ----------
        team_member_id : str
            The ID of the team member for which to retrieve the wage setting.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetWageSettingResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/team-members/{jsonable_encoder(team_member_id)}/wage-setting",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetWageSettingResponse,
                    construct_type(
                        type_=GetWageSettingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        team_member_id: str,
        *,
        wage_setting: WageSettingParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateWageSettingResponse]:
        """
        Creates or updates a `WageSetting` object. The object is created if a
        `WageSetting` with the specified `team_member_id` doesn't exist. Otherwise,
        it fully replaces the `WageSetting` object for the team member.
        The `WageSetting` is returned on a successful update. For more information, see
        [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#create-or-update-a-wage-setting).

        Square recommends using [CreateTeamMember](api-endpoint:Team-CreateTeamMember) or [UpdateTeamMember](api-endpoint:Team-UpdateTeamMember)
        to manage the `TeamMember.wage_setting` field directly.

        Parameters
        ----------
        team_member_id : str
            The ID of the team member for which to update the `WageSetting` object.

        wage_setting : WageSettingParams
            The complete `WageSetting` object. For all job assignments, specify one of the following:
            - `job_id` (recommended) - If needed, call [ListJobs](api-endpoint:Team-ListJobs) to get a list of all jobs.
            Requires Square API version 2024-12-18 or later.
            - `job_title` - Use the exact, case-sensitive spelling of an existing title unless you want to create a new job.
            This value is ignored if `job_id` is also provided.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateWageSettingResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/team-members/{jsonable_encoder(team_member_id)}/wage-setting",
            method="PUT",
            json={
                "wage_setting": convert_and_respect_annotation_metadata(
                    object_=wage_setting, annotation=WageSettingParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateWageSettingResponse,
                    construct_type(
                        type_=UpdateWageSettingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawWageSettingClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, team_member_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetWageSettingResponse]:
        """
        Retrieves a `WageSetting` object for a team member specified
        by `TeamMember.id`. For more information, see
        [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#retrievewagesetting).

        Square recommends using [RetrieveTeamMember](api-endpoint:Team-RetrieveTeamMember) or [SearchTeamMembers](api-endpoint:Team-SearchTeamMembers)
        to get this information directly from the `TeamMember.wage_setting` field.

        Parameters
        ----------
        team_member_id : str
            The ID of the team member for which to retrieve the wage setting.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetWageSettingResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/team-members/{jsonable_encoder(team_member_id)}/wage-setting",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetWageSettingResponse,
                    construct_type(
                        type_=GetWageSettingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        team_member_id: str,
        *,
        wage_setting: WageSettingParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateWageSettingResponse]:
        """
        Creates or updates a `WageSetting` object. The object is created if a
        `WageSetting` with the specified `team_member_id` doesn't exist. Otherwise,
        it fully replaces the `WageSetting` object for the team member.
        The `WageSetting` is returned on a successful update. For more information, see
        [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#create-or-update-a-wage-setting).

        Square recommends using [CreateTeamMember](api-endpoint:Team-CreateTeamMember) or [UpdateTeamMember](api-endpoint:Team-UpdateTeamMember)
        to manage the `TeamMember.wage_setting` field directly.

        Parameters
        ----------
        team_member_id : str
            The ID of the team member for which to update the `WageSetting` object.

        wage_setting : WageSettingParams
            The complete `WageSetting` object. For all job assignments, specify one of the following:
            - `job_id` (recommended) - If needed, call [ListJobs](api-endpoint:Team-ListJobs) to get a list of all jobs.
            Requires Square API version 2024-12-18 or later.
            - `job_title` - Use the exact, case-sensitive spelling of an existing title unless you want to create a new job.
            This value is ignored if `job_id` is also provided.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateWageSettingResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/team-members/{jsonable_encoder(team_member_id)}/wage-setting",
            method="PUT",
            json={
                "wage_setting": convert_and_respect_annotation_metadata(
                    object_=wage_setting, annotation=WageSettingParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateWageSettingResponse,
                    construct_type(
                        type_=UpdateWageSettingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .order_quantity_unit import OrderQuantityUnitParams
from ..types.order_line_item_item_type import OrderLineItemItemType
from .order_line_item_modifier import OrderLineItemModifierParams
from .order_line_item_applied_tax import OrderLineItemAppliedTaxParams
from .order_line_item_applied_discount import OrderLineItemAppliedDiscountParams
from .order_line_item_applied_service_charge import OrderLineItemAppliedServiceChargeParams
from .money import MoneyParams
from .order_line_item_pricing_blocklists import OrderLineItemPricingBlocklistsParams


class OrderLineItemParams(typing_extensions.TypedDict):
    """
    Represents a line item in an order. Each line item describes a different
    product to purchase, with its own quantity and price details.
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID that identifies the line item only within this order.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the line item.
    """

    quantity: str
    """
    The count, or measurement, of a line item being purchased:
    
    If `quantity` is a whole number, and `quantity_unit` is not specified, then `quantity` denotes an item count.  For example: `3` apples.
    
    If `quantity` is a whole or decimal number, and `quantity_unit` is also specified, then `quantity` denotes a measurement.  For example: `2.25` pounds of broccoli.
    
    For more information, see [Specify item quantity and measurement unit](https://developer.squareup.com/docs/orders-api/create-orders#specify-item-quantity-and-measurement-unit).
    
    Line items with a quantity of `0` are automatically removed
    when paying for or otherwise completing the order.
    """

    quantity_unit: typing_extensions.NotRequired[OrderQuantityUnitParams]
    """
    The measurement unit and decimal precision that this line item's quantity is measured in.
    """

    note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    An optional note associated with the line item.
    """

    catalog_object_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The [CatalogItemVariation](entity:CatalogItemVariation) ID applied to this line item.
    """

    catalog_version: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The version of the catalog object that this line item references.
    """

    variation_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the variation applied to this line item.
    """

    item_type: typing_extensions.NotRequired[OrderLineItemItemType]
    """
    The type of line item: an itemized sale, a non-itemized sale (custom amount), or the
    activation or reloading of a gift card.
    See [OrderLineItemItemType](#type-orderlineitemitemtype) for possible values
    """

    metadata: typing_extensions.NotRequired[typing.Optional[typing.Dict[str, typing.Optional[str]]]]
    """
    Application-defined data attached to this line item. Metadata fields are intended
    to store descriptive references or associations with an entity in another system or store brief
    information about the object. Square does not process this field; it only stores and returns it
    in relevant API calls. Do not use metadata to store any sensitive information (such as personally
    identifiable information or card details).
    
    Keys written by applications must be 60 characters or less and must be in the character set
    `[a-zA-Z0-9_-]`. Entries can also include metadata generated by Square. These keys are prefixed
    with a namespace, separated from the key with a ':' character.
    
    Values have a maximum length of 255 characters.
    
    An application can have up to 10 entries per metadata field.
    
    Entries written by applications are private and can only be read or modified by the same
    application.
    
    For more information, see [Metadata](https://developer.squareup.com/docs/build-basics/metadata).
    """

    modifiers: typing_extensions.NotRequired[typing.Optional[typing.Sequence[OrderLineItemModifierParams]]]
    """
    The [CatalogModifier](entity:CatalogModifier)s applied to this line item.
    """

    applied_taxes: typing_extensions.NotRequired[typing.Optional[typing.Sequence[OrderLineItemAppliedTaxParams]]]
    """
    The list of references to taxes applied to this line item. Each
    `OrderLineItemAppliedTax` has a `tax_uid` that references the `uid` of a
    top-level `OrderLineItemTax` applied to the line item. On reads, the
    amount applied is populated.
    
    An `OrderLineItemAppliedTax` is automatically created on every line
    item for all `ORDER` scoped taxes added to the order. `OrderLineItemAppliedTax`
    records for `LINE_ITEM` scoped taxes must be added in requests for the tax
    to apply to any line items.
    
    To change the amount of a tax, modify the referenced top-level tax.
    """

    applied_discounts: typing_extensions.NotRequired[
        typing.Optional[typing.Sequence[OrderLineItemAppliedDiscountParams]]
    ]
    """
    The list of references to discounts applied to this line item. Each
    `OrderLineItemAppliedDiscount` has a `discount_uid` that references the `uid` of a top-level
    `OrderLineItemDiscounts` applied to the line item. On reads, the amount
    applied is populated.
    
    An `OrderLineItemAppliedDiscount` is automatically created on every line item for all
    `ORDER` scoped discounts that are added to the order. `OrderLineItemAppliedDiscount` records
    for `LINE_ITEM` scoped discounts must be added in requests for the discount to apply to any
    line items.
    
    To change the amount of a discount, modify the referenced top-level discount.
    """

    applied_service_charges: typing_extensions.NotRequired[
        typing.Optional[typing.Sequence[OrderLineItemAppliedServiceChargeParams]]
    ]
    """
    The list of references to service charges applied to this line item. Each
    `OrderLineItemAppliedServiceCharge` has a `service_charge_id` that references the `uid` of a
    top-level `OrderServiceCharge` applied to the line item. On reads, the amount applied is
    populated.
    
    To change the amount of a service charge, modify the referenced top-level service charge.
    """

    base_price_money: typing_extensions.NotRequired[MoneyParams]
    """
    The base price for a single unit of the line item.
    """

    variation_total_price_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total price of all item variations sold in this line item.
    The price is calculated as `base_price_money` multiplied by `quantity`.
    It does not include modifiers.
    """

    gross_sales_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money made in gross sales for this line item.
    The amount is calculated as the sum of the variation's total price and each modifier's total price.
    For inclusive tax items in the US, Canada, and Japan, tax is deducted from `gross_sales_money`. For Europe and
    Australia, inclusive tax remains as part of the gross sale calculation.
    """

    total_tax_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of tax money to collect for the line item.
    """

    total_discount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of discount money to collect for the line item.
    """

    total_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of money to collect for this line item.
    """

    pricing_blocklists: typing_extensions.NotRequired[OrderLineItemPricingBlocklistsParams]
    """
    Describes pricing adjustments that are blocked from automatic
    application to a line item. For more information, see
    [Apply Taxes and Discounts](https://developer.squareup.com/docs/orders-api/apply-taxes-and-discounts).
    """

    total_service_charge_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of apportioned service charge money to collect for the line item.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .transaction import TransactionParams


class ListTransactionsResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the [ListTransactions](api-endpoint:Transactions-ListTransactions) endpoint.

    One of `errors` or `transactions` is present in a given response (never both).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    transactions: typing_extensions.NotRequired[typing.Sequence[TransactionParams]]
    """
    An array of transactions that match your query.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    A pagination cursor for retrieving the next set of results,
    if any remain. Provide this value as the `cursor` parameter in a subsequent
    request to this endpoint.
    
    See [Paginating results](https://developer.squareup.com/docs/working-with-apis/pagination) for more information.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
import typing
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.get_device_response import GetDeviceResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse


class RawDevicesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, device_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetDeviceResponse]:
        """
        Retrieves Device with the associated `device_id`.

        Parameters
        ----------
        device_id : str
            The unique ID for the desired `Device`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetDeviceResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/devices/{jsonable_encoder(device_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetDeviceResponse,
                    construct_type(
                        type_=GetDeviceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawDevicesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, device_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetDeviceResponse]:
        """
        Retrieves Device with the associated `device_id`.

        Parameters
        ----------
        device_id : str
            The unique ID for the desired `Device`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetDeviceResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/devices/{jsonable_encoder(device_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetDeviceResponse,
                    construct_type(
                        type_=GetDeviceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .currency import Currency
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class V1Money(UncheckedBaseModel):
    amount: typing.Optional[int] = pydantic.Field(default=None)
    """
    Amount in the lowest denominated value of this Currency. E.g. in USD
    these are cents, in JPY they are Yen (which do not have a 'cent' concept).
    """

    currency_code: typing.Optional[Currency] = pydantic.Field(default=None)
    """
    
    See [Currency](#type-currency) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .fulfillment_recipient import FulfillmentRecipient
import pydantic
from .fulfillment_delivery_details_order_fulfillment_delivery_details_schedule_type import (
    FulfillmentDeliveryDetailsOrderFulfillmentDeliveryDetailsScheduleType,
)
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class FulfillmentDeliveryDetails(UncheckedBaseModel):
    """
    Describes delivery details of an order fulfillment.
    """

    recipient: typing.Optional[FulfillmentRecipient] = pydantic.Field(default=None)
    """
    The contact information for the person to receive the fulfillment.
    """

    schedule_type: typing.Optional[FulfillmentDeliveryDetailsOrderFulfillmentDeliveryDetailsScheduleType] = (
        pydantic.Field(default=None)
    )
    """
    Indicates the fulfillment delivery schedule type. If `SCHEDULED`, then
    `deliver_at` is required. If `ASAP`, then `prep_time_duration` is required. The default is `SCHEDULED`.
    See [OrderFulfillmentDeliveryDetailsScheduleType](#type-orderfulfillmentdeliverydetailsscheduletype) for possible values
    """

    placed_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the fulfillment was placed.
    The timestamp must be in RFC 3339 format (for example, "2016-09-04T23:59:33.123Z").
    
    Must be in RFC 3339 timestamp format, e.g., "2016-09-04T23:59:33.123Z".
    """

    deliver_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    that represents the start of the delivery period.
    When the fulfillment `schedule_type` is `ASAP`, the field is automatically
    set to the current time plus the `prep_time_duration`.
    Otherwise, the application can set this field while the fulfillment `state` is
    `PROPOSED`, `RESERVED`, or `PREPARED` (any time before the
    terminal state such as `COMPLETED`, `CANCELED`, and `FAILED`).
    
    The timestamp must be in RFC 3339 format
    (for example, "2016-09-04T23:59:33.123Z").
    """

    prep_time_duration: typing.Optional[str] = pydantic.Field(default=None)
    """
    The duration of time it takes to prepare and deliver this fulfillment.
    The duration must be in RFC 3339 format (for example, "P1W3D").
    """

    delivery_window_duration: typing.Optional[str] = pydantic.Field(default=None)
    """
    The time period after `deliver_at` in which to deliver the order.
    Applications can set this field when the fulfillment `state` is
    `PROPOSED`, `RESERVED`, or `PREPARED` (any time before the terminal state
    such as `COMPLETED`, `CANCELED`, and `FAILED`).
    
    The duration must be in RFC 3339 format (for example, "P1W3D").
    """

    note: typing.Optional[str] = pydantic.Field(default=None)
    """
    Provides additional instructions about the delivery fulfillment.
    It is displayed in the Square Point of Sale application and set by the API.
    """

    completed_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicates when the seller completed the fulfillment.
    This field is automatically set when  fulfillment `state` changes to `COMPLETED`.
    The timestamp must be in RFC 3339 format (for example, "2016-09-04T23:59:33.123Z").
    """

    in_progress_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicates when the seller started processing the fulfillment.
    This field is automatically set when the fulfillment `state` changes to `RESERVED`.
    The timestamp must be in RFC 3339 format (for example, "2016-09-04T23:59:33.123Z").
    """

    rejected_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the fulfillment was rejected. This field is
    automatically set when the fulfillment `state` changes to `FAILED`.
    The timestamp must be in RFC 3339 format (for example, "2016-09-04T23:59:33.123Z").
    """

    ready_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the seller marked the fulfillment as ready for
    courier pickup. This field is automatically set when the fulfillment `state` changes
    to PREPARED.
    The timestamp must be in RFC 3339 format (for example, "2016-09-04T23:59:33.123Z").
    """

    delivered_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the fulfillment was delivered to the recipient.
    The timestamp must be in RFC 3339 format (for example, "2016-09-04T23:59:33.123Z").
    """

    canceled_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the fulfillment was canceled. This field is automatically
    set when the fulfillment `state` changes to `CANCELED`.
    
    The timestamp must be in RFC 3339 format (for example, "2016-09-04T23:59:33.123Z").
    """

    cancel_reason: typing.Optional[str] = pydantic.Field(default=None)
    """
    The delivery cancellation reason. Max length: 100 characters.
    """

    courier_pickup_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when an order can be picked up by the courier for delivery.
    The timestamp must be in RFC 3339 format (for example, "2016-09-04T23:59:33.123Z").
    """

    courier_pickup_window_duration: typing.Optional[str] = pydantic.Field(default=None)
    """
    The time period after `courier_pickup_at` in which the courier should pick up the order.
    The duration must be in RFC 3339 format (for example, "P1W3D").
    """

    is_no_contact_delivery: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the delivery is preferred to be no contact.
    """

    dropoff_notes: typing.Optional[str] = pydantic.Field(default=None)
    """
    A note to provide additional instructions about how to deliver the order.
    """

    courier_provider_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the courier provider.
    """

    courier_support_phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The support phone number of the courier.
    """

    square_delivery_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The identifier for the delivery created by Square.
    """

    external_delivery_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The identifier for the delivery created by the third-party courier service.
    """

    managed_delivery: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The flag to indicate the delivery is managed by a third party (ie DoorDash), which means
    we may not receive all recipient information for PII purposes.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .employee_wage import EmployeeWageParams
from .error import ErrorParams


class ListEmployeeWagesResponseParams(typing_extensions.TypedDict):
    """
    The response to a request for a set of `EmployeeWage` objects. The response contains
    a set of `EmployeeWage` objects.
    """

    employee_wages: typing_extensions.NotRequired[typing.Sequence[EmployeeWageParams]]
    """
    A page of `EmployeeWage` results.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The value supplied in the subsequent request to fetch the next page
    of `EmployeeWage` results.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.activity_type import ActivityType
from .money import MoneyParams
from .payment_balance_activity_app_fee_revenue_detail import PaymentBalanceActivityAppFeeRevenueDetailParams
from .payment_balance_activity_app_fee_refund_detail import PaymentBalanceActivityAppFeeRefundDetailParams
from .payment_balance_activity_automatic_savings_detail import PaymentBalanceActivityAutomaticSavingsDetailParams
from .payment_balance_activity_automatic_savings_reversed_detail import (
    PaymentBalanceActivityAutomaticSavingsReversedDetailParams,
)
from .payment_balance_activity_charge_detail import PaymentBalanceActivityChargeDetailParams
from .payment_balance_activity_deposit_fee_detail import PaymentBalanceActivityDepositFeeDetailParams
from .payment_balance_activity_deposit_fee_reversed_detail import PaymentBalanceActivityDepositFeeReversedDetailParams
from .payment_balance_activity_dispute_detail import PaymentBalanceActivityDisputeDetailParams
from .payment_balance_activity_fee_detail import PaymentBalanceActivityFeeDetailParams
from .payment_balance_activity_free_processing_detail import PaymentBalanceActivityFreeProcessingDetailParams
from .payment_balance_activity_hold_adjustment_detail import PaymentBalanceActivityHoldAdjustmentDetailParams
from .payment_balance_activity_open_dispute_detail import PaymentBalanceActivityOpenDisputeDetailParams
from .payment_balance_activity_other_detail import PaymentBalanceActivityOtherDetailParams
from .payment_balance_activity_other_adjustment_detail import PaymentBalanceActivityOtherAdjustmentDetailParams
from .payment_balance_activity_refund_detail import PaymentBalanceActivityRefundDetailParams
from .payment_balance_activity_release_adjustment_detail import PaymentBalanceActivityReleaseAdjustmentDetailParams
from .payment_balance_activity_reserve_hold_detail import PaymentBalanceActivityReserveHoldDetailParams
from .payment_balance_activity_reserve_release_detail import PaymentBalanceActivityReserveReleaseDetailParams
from .payment_balance_activity_square_capital_payment_detail import (
    PaymentBalanceActivitySquareCapitalPaymentDetailParams,
)
from .payment_balance_activity_square_capital_reversed_payment_detail import (
    PaymentBalanceActivitySquareCapitalReversedPaymentDetailParams,
)
from .payment_balance_activity_tax_on_fee_detail import PaymentBalanceActivityTaxOnFeeDetailParams
from .payment_balance_activity_third_party_fee_detail import PaymentBalanceActivityThirdPartyFeeDetailParams
from .payment_balance_activity_third_party_fee_refund_detail import (
    PaymentBalanceActivityThirdPartyFeeRefundDetailParams,
)
from .payment_balance_activity_square_payroll_transfer_detail import (
    PaymentBalanceActivitySquarePayrollTransferDetailParams,
)
from .payment_balance_activity_square_payroll_transfer_reversed_detail import (
    PaymentBalanceActivitySquarePayrollTransferReversedDetailParams,
)


class PayoutEntryParams(typing_extensions.TypedDict):
    """
    One or more PayoutEntries that make up a Payout. Each one has a date, amount, and type of activity.
    The total amount of the payout will equal the sum of the payout entries for a batch payout
    """

    id: str
    """
    A unique ID for the payout entry.
    """

    payout_id: str
    """
    The ID of the payout entries’ associated payout.
    """

    effective_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The timestamp of when the payout entry affected the balance, in RFC 3339 format.
    """

    type: typing_extensions.NotRequired[ActivityType]
    """
    The type of activity associated with this payout entry.
    See [ActivityType](#type-activitytype) for possible values
    """

    gross_amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money involved in this payout entry.
    """

    fee_amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of Square fees associated with this payout entry.
    """

    net_amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The net proceeds from this transaction after any fees.
    """

    type_app_fee_revenue_details: typing_extensions.NotRequired[PaymentBalanceActivityAppFeeRevenueDetailParams]
    """
    Details of any developer app fee revenue generated on a payment.
    """

    type_app_fee_refund_details: typing_extensions.NotRequired[PaymentBalanceActivityAppFeeRefundDetailParams]
    """
    Details of a refund for an app fee on a payment.
    """

    type_automatic_savings_details: typing_extensions.NotRequired[PaymentBalanceActivityAutomaticSavingsDetailParams]
    """
    Details of any automatic transfer from the payment processing balance to the Square Savings account. These are, generally, proportional to the merchant's sales.
    """

    type_automatic_savings_reversed_details: typing_extensions.NotRequired[
        PaymentBalanceActivityAutomaticSavingsReversedDetailParams
    ]
    """
    Details of any automatic transfer from the Square Savings account back to the processing balance. These are, generally, proportional to the merchant's refunds.
    """

    type_charge_details: typing_extensions.NotRequired[PaymentBalanceActivityChargeDetailParams]
    """
    Details of credit card payment captures.
    """

    type_deposit_fee_details: typing_extensions.NotRequired[PaymentBalanceActivityDepositFeeDetailParams]
    """
    Details of any fees involved with deposits such as for instant deposits.
    """

    type_deposit_fee_reversed_details: typing_extensions.NotRequired[
        PaymentBalanceActivityDepositFeeReversedDetailParams
    ]
    """
    Details of any reversal or refund of fees involved with deposits such as for instant deposits.
    """

    type_dispute_details: typing_extensions.NotRequired[PaymentBalanceActivityDisputeDetailParams]
    """
    Details of any balance change due to a dispute event.
    """

    type_fee_details: typing_extensions.NotRequired[PaymentBalanceActivityFeeDetailParams]
    """
    Details of adjustments due to the Square processing fee.
    """

    type_free_processing_details: typing_extensions.NotRequired[PaymentBalanceActivityFreeProcessingDetailParams]
    """
    Square offers Free Payments Processing for a variety of business scenarios including seller referral or when Square wants to apologize for a bug, customer service, repricing complication, and so on. This entry represents details of any credit to the merchant for the purposes of Free Processing.
    """

    type_hold_adjustment_details: typing_extensions.NotRequired[PaymentBalanceActivityHoldAdjustmentDetailParams]
    """
    Details of any adjustment made by Square related to the holding or releasing of a payment.
    """

    type_open_dispute_details: typing_extensions.NotRequired[PaymentBalanceActivityOpenDisputeDetailParams]
    """
    Details of any open disputes.
    """

    type_other_details: typing_extensions.NotRequired[PaymentBalanceActivityOtherDetailParams]
    """
    Details of any other type that does not belong in the rest of the types.
    """

    type_other_adjustment_details: typing_extensions.NotRequired[PaymentBalanceActivityOtherAdjustmentDetailParams]
    """
    Details of any other type of adjustments that don't fall under existing types.
    """

    type_refund_details: typing_extensions.NotRequired[PaymentBalanceActivityRefundDetailParams]
    """
    Details of a refund for an existing card payment.
    """

    type_release_adjustment_details: typing_extensions.NotRequired[PaymentBalanceActivityReleaseAdjustmentDetailParams]
    """
    Details of fees released for adjustments.
    """

    type_reserve_hold_details: typing_extensions.NotRequired[PaymentBalanceActivityReserveHoldDetailParams]
    """
    Details of fees paid for funding risk reserve.
    """

    type_reserve_release_details: typing_extensions.NotRequired[PaymentBalanceActivityReserveReleaseDetailParams]
    """
    Details of fees released from risk reserve.
    """

    type_square_capital_payment_details: typing_extensions.NotRequired[
        PaymentBalanceActivitySquareCapitalPaymentDetailParams
    ]
    """
    Details of capital merchant cash advance (MCA) assessments. These are, generally, proportional to the merchant's sales but may be issued for other reasons related to the MCA.
    """

    type_square_capital_reversed_payment_details: typing_extensions.NotRequired[
        PaymentBalanceActivitySquareCapitalReversedPaymentDetailParams
    ]
    """
    Details of capital merchant cash advance (MCA) assessment refunds. These are, generally, proportional to the merchant's refunds but may be issued for other reasons related to the MCA.
    """

    type_tax_on_fee_details: typing_extensions.NotRequired[PaymentBalanceActivityTaxOnFeeDetailParams]
    """
    Details of tax paid on fee amounts.
    """

    type_third_party_fee_details: typing_extensions.NotRequired[PaymentBalanceActivityThirdPartyFeeDetailParams]
    """
    Details of fees collected by a 3rd party platform.
    """

    type_third_party_fee_refund_details: typing_extensions.NotRequired[
        PaymentBalanceActivityThirdPartyFeeRefundDetailParams
    ]
    """
    Details of refunded fees from a 3rd party platform.
    """

    type_square_payroll_transfer_details: typing_extensions.NotRequired[
        PaymentBalanceActivitySquarePayrollTransferDetailParams
    ]
    """
    Details of a payroll payment that was transferred to a team member’s bank account.
    """

    type_square_payroll_transfer_reversed_details: typing_extensions.NotRequired[
        PaymentBalanceActivitySquarePayrollTransferReversedDetailParams
    ]
    """
    Details of a payroll payment to a team member’s bank account that was deposited back to the seller’s account by Square.
    """



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
import typing_extensions
import typing_extensions
import typing
from ..types.catalog_modifier_list_selection_type import CatalogModifierListSelectionType
from ..types.catalog_modifier_list_modifier_type import CatalogModifierListModifierType
import typing

if typing.TYPE_CHECKING:
    from .catalog_object import CatalogObjectParams


class CatalogModifierListParams(typing_extensions.TypedDict):
    """
    A container for a list of modifiers, or a text-based modifier.
    For text-based modifiers, this represents text configuration for an item. (For example, custom text to print on a t-shirt).
    For non text-based modifiers, this represents a list of modifiers that can be applied to items at the time of sale.
    (For example, a list of condiments for a hot dog, or a list of ice cream flavors).
    Each element of the modifier list is a `CatalogObject` instance of the `MODIFIER` type.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the `CatalogModifierList` instance. This is a searchable attribute for use in applicable query filters, and its value length is of 
    Unicode code points.
    """

    ordinal: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The position of this `CatalogModifierList` within a list of `CatalogModifierList` instances.
    """

    selection_type: typing_extensions.NotRequired[CatalogModifierListSelectionType]
    """
    __Deprecated__: Indicates whether a single (`SINGLE`) modifier or multiple (`MULTIPLE`) modifiers can be selected. Use
    `min_selected_modifiers` and `max_selected_modifiers` instead.
    See [CatalogModifierListSelectionType](#type-catalogmodifierlistselectiontype) for possible values
    """

    modifiers: typing_extensions.NotRequired[typing.Optional[typing.Sequence["CatalogObjectParams"]]]
    """
    A non-empty list of `CatalogModifier` objects to be included in the `CatalogModifierList`, 
    for non text-based modifiers when the `modifier_type` attribute is `LIST`. Each element of this list 
    is a `CatalogObject` instance of the `MODIFIER` type, containing the following attributes:
    ```
    {
    "id": "{{catalog_modifier_id}}",
    "type": "MODIFIER", 
    "modifier_data": {{a CatalogModifier instance>}} 
    }
    ```
    """

    image_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The IDs of images associated with this `CatalogModifierList` instance.
    Currently these images are not displayed on Square products, but may be displayed in 3rd-party applications.
    """

    allow_quantities: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    When `true`, allows multiple quantities of the same modifier to be selected.
    """

    is_conversational: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    True if modifiers belonging to this list can be used conversationally.
    """

    modifier_type: typing_extensions.NotRequired[CatalogModifierListModifierType]
    """
    The type of the modifier. 
    
    When this `modifier_type` value is `TEXT`,  the `CatalogModifierList` represents a text-based modifier. 
    When this `modifier_type` value is `LIST`, the `CatalogModifierList` contains a list of `CatalogModifier` objects.
    See [CatalogModifierListModifierType](#type-catalogmodifierlistmodifiertype) for possible values
    """

    max_length: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum length, in Unicode points, of the text string of the text-based modifier as represented by 
    this `CatalogModifierList` object with the `modifier_type` set to `TEXT`.
    """

    text_required: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Whether the text string must be a non-empty string (`true`) or not (`false`) for a text-based modifier
    as represented by this `CatalogModifierList` object with the `modifier_type` set to `TEXT`.
    """

    internal_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A note for internal use by the business.   
    
    For example, for a text-based modifier applied to a T-shirt item, if the buyer-supplied text of "Hello, Kitty!"  
    is to be printed on the T-shirt, this `internal_name` attribute can be "Use italic face" as 
    an instruction for the business to follow.  
    
    For non text-based modifiers, this `internal_name` attribute can be 
    used to include SKUs, internal codes, or supplemental descriptions for internal use.
    """

    min_selected_modifiers: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The minimum number of modifiers that must be selected from this list. The value can be overridden with `CatalogItemModifierListInfo`.
    
    Values:
    
    - 0: No selection is required.
    - -1: Default value, the attribute was not set by the client. Treated as no selection required.
    - &gt;0: The required minimum modifier selections. This can be larger than the total `CatalogModifiers` when `allow_quantities` is enabled.
    - &lt; -1: Invalid. Treated as no selection required.
    """

    max_selected_modifiers: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum number of modifiers that must be selected from this list. The value can be overridden with `CatalogItemModifierListInfo`.
    
    Values:
    
    - 0: No maximum limit.
    - -1: Default value, the attribute was not set by the client. Treated as no maximum limit.
    - &gt;0: The maximum total modifier selections. This can be larger than the total `CatalogModifiers` when `allow_quantities` is enabled.
    - &lt; -1: Invalid. Treated as no maximum limit.
    """

    hidden_from_customer: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    If `true`, modifiers from this list are hidden from customer receipts. The default value is `false`.
    This setting can be overridden with `CatalogItemModifierListInfo.hidden_from_customer_override`.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .customer_custom_attribute_filter_value import CustomerCustomAttributeFilterValueParams
from .time_range import TimeRangeParams


class CustomerCustomAttributeFilterParams(typing_extensions.TypedDict):
    """
    The custom attribute filter. Use this filter in a set of [custom attribute filters](entity:CustomerCustomAttributeFilters) to search
    based on the value or last updated date of a customer-related [custom attribute](entity:CustomAttribute).
    """

    key: str
    """
    The `key` of the [custom attribute](entity:CustomAttribute) to filter by. The key is the identifier of the custom attribute
    (and the corresponding custom attribute definition) and can be retrieved using the [Customer Custom Attributes API](api:CustomerCustomAttributes).
    """

    filter: typing_extensions.NotRequired[CustomerCustomAttributeFilterValueParams]
    """
    A filter that corresponds to the data type of the target custom attribute. For example, provide the `phone` filter to
    search based on the value of a `PhoneNumber`-type custom attribute. The data type is specified by the schema field of the custom attribute definition,
    which can be retrieved using the [Customer Custom Attributes API](api:CustomerCustomAttributes).
    
    You must provide this `filter` field, the `updated_at` field, or both.
    """

    updated_at: typing_extensions.NotRequired[TimeRangeParams]
    """
    The date range for when the custom attribute was last updated. The date range can include `start_at`, `end_at`, or
    both. Range boundaries are inclusive. Dates are specified as RFC 3339 timestamps.
    
    You must provide this `updated_at` field, the `filter` field, or both.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .loyalty_reward import LoyaltyRewardParams


class SearchLoyaltyRewardsResponseParams(typing_extensions.TypedDict):
    """
    A response that includes the loyalty rewards satisfying the search criteria.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    rewards: typing_extensions.NotRequired[typing.Sequence[LoyaltyRewardParams]]
    """
    The loyalty rewards that satisfy the search criteria.
    These are returned in descending order by `updated_at`.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to be used in a subsequent 
    request. If empty, this is the final response.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .catalog_quick_amounts_settings_option import CatalogQuickAmountsSettingsOption
import pydantic
import typing
from .catalog_quick_amount import CatalogQuickAmount
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogQuickAmountsSettings(UncheckedBaseModel):
    """
    A parent Catalog Object model represents a set of Quick Amounts and the settings control the amounts.
    """

    option: CatalogQuickAmountsSettingsOption = pydantic.Field()
    """
    Represents the option seller currently uses on Quick Amounts.
    See [CatalogQuickAmountsSettingsOption](#type-catalogquickamountssettingsoption) for possible values
    """

    eligible_for_auto_amounts: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Represents location's eligibility for auto amounts
    The boolean should be consistent with whether there are AUTO amounts in the `amounts`.
    """

    amounts: typing.Optional[typing.List[CatalogQuickAmount]] = pydantic.Field(default=None)
    """
    Represents a set of Quick Amounts at this location.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import SyncClientWrapper
import typing
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.list_webhook_event_types_response import ListWebhookEventTypesResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse


class RawEventTypesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, api_version: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ListWebhookEventTypesResponse]:
        """
        Lists all webhook event types that can be subscribed to.

        Parameters
        ----------
        api_version : typing.Optional[str]
            The API version for which to list event types. Setting this field overrides the default version used by the application.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListWebhookEventTypesResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/webhooks/event-types",
            method="GET",
            params={
                "api_version": api_version,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListWebhookEventTypesResponse,
                    construct_type(
                        type_=ListWebhookEventTypesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawEventTypesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, api_version: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ListWebhookEventTypesResponse]:
        """
        Lists all webhook event types that can be subscribed to.

        Parameters
        ----------
        api_version : typing.Optional[str]
            The API version for which to list event types. Setting this field overrides the default version used by the application.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListWebhookEventTypesResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/webhooks/event-types",
            method="GET",
            params={
                "api_version": api_version,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListWebhookEventTypesResponse,
                    construct_type(
                        type_=ListWebhookEventTypesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..requests.card import CardParams
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.create_card_response import CreateCardResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.get_card_response import GetCardResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..types.disable_card_response import DisableCardResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCardsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        idempotency_key: str,
        source_id: str,
        card: CardParams,
        verification_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateCardResponse]:
        """
        Adds a card on file to an existing merchant.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this CreateCard request. Keys can be
            any valid string and must be unique for every request.

            Max: 45 characters

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        source_id : str
            The ID of the source which represents the card information to be stored. This can be a card nonce or a payment id.

        card : CardParams
            Payment details associated with the card to be stored.

        verification_token : typing.Optional[str]
            An identifying token generated by [Payments.verifyBuyer()](https://developer.squareup.com/reference/sdks/web/payments/objects/Payments#Payments.verifyBuyer).
            Verification tokens encapsulate customer device information and 3-D Secure
            challenge results to indicate that Square has verified the buyer identity.

            See the [SCA Overview](https://developer.squareup.com/docs/sca-overview).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateCardResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/cards",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "source_id": source_id,
                "verification_token": verification_token,
                "card": convert_and_respect_annotation_metadata(object_=card, annotation=CardParams, direction="write"),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateCardResponse,
                    construct_type(
                        type_=CreateCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, card_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetCardResponse]:
        """
        Retrieves details for a specific Card.

        Parameters
        ----------
        card_id : str
            Unique ID for the desired Card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetCardResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/cards/{jsonable_encoder(card_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetCardResponse,
                    construct_type(
                        type_=GetCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def disable(
        self, card_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DisableCardResponse]:
        """
        Disables the card, preventing any further updates or charges.
        Disabling an already disabled card is allowed but has no effect.

        Parameters
        ----------
        card_id : str
            Unique ID for the desired Card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DisableCardResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/cards/{jsonable_encoder(card_id)}/disable",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DisableCardResponse,
                    construct_type(
                        type_=DisableCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawCardsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        idempotency_key: str,
        source_id: str,
        card: CardParams,
        verification_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateCardResponse]:
        """
        Adds a card on file to an existing merchant.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this CreateCard request. Keys can be
            any valid string and must be unique for every request.

            Max: 45 characters

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        source_id : str
            The ID of the source which represents the card information to be stored. This can be a card nonce or a payment id.

        card : CardParams
            Payment details associated with the card to be stored.

        verification_token : typing.Optional[str]
            An identifying token generated by [Payments.verifyBuyer()](https://developer.squareup.com/reference/sdks/web/payments/objects/Payments#Payments.verifyBuyer).
            Verification tokens encapsulate customer device information and 3-D Secure
            challenge results to indicate that Square has verified the buyer identity.

            See the [SCA Overview](https://developer.squareup.com/docs/sca-overview).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateCardResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/cards",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "source_id": source_id,
                "verification_token": verification_token,
                "card": convert_and_respect_annotation_metadata(object_=card, annotation=CardParams, direction="write"),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateCardResponse,
                    construct_type(
                        type_=CreateCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, card_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetCardResponse]:
        """
        Retrieves details for a specific Card.

        Parameters
        ----------
        card_id : str
            Unique ID for the desired Card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetCardResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/cards/{jsonable_encoder(card_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetCardResponse,
                    construct_type(
                        type_=GetCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def disable(
        self, card_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DisableCardResponse]:
        """
        Disables the card, preventing any further updates or charges.
        Disabling an already disabled card is allowed but has no effect.

        Parameters
        ----------
        card_id : str
            Unique ID for the desired Card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DisableCardResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/cards/{jsonable_encoder(card_id)}/disable",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DisableCardResponse,
                    construct_type(
                        type_=DisableCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .device_component_details_external_power import DeviceComponentDetailsExternalPower
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DeviceComponentDetailsBatteryDetails(UncheckedBaseModel):
    visible_percent: typing.Optional[int] = pydantic.Field(default=None)
    """
    The battery charge percentage as displayed on the device.
    """

    external_power: typing.Optional[DeviceComponentDetailsExternalPower] = pydantic.Field(default=None)
    """
    The status of external_power.
    See [ExternalPower](#type-externalpower) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class InvoiceFilter(UncheckedBaseModel):
    """
    Describes query filters to apply.
    """

    location_ids: typing.List[str] = pydantic.Field()
    """
    Limits the search to the specified locations. A location is required. 
    In the current implementation, only one location can be specified.
    """

    customer_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Limits the search to the specified customers, within the specified locations. 
    Specifying a customer is optional. In the current implementation, 
    a maximum of one customer can be specified.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .shift_filter import ShiftFilter
import pydantic
from .shift_sort import ShiftSort
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ShiftQuery(UncheckedBaseModel):
    """
    The parameters of a `Shift` search query, which includes filter and sort options.

    Deprecated at Square API version 2025-05-21. See the [migration notes](https://developer.squareup.com/docs/labor-api/what-it-does#migration-notes).
    """

    filter: typing.Optional[ShiftFilter] = pydantic.Field(default=None)
    """
    Query filter options.
    """

    sort: typing.Optional[ShiftSort] = pydantic.Field(default=None)
    """
    Sort order details.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from . import (
    apple_pay,
    bank_accounts,
    bookings,
    cards,
    cash_drawers,
    catalog,
    checkout,
    customers,
    devices,
    disputes,
    employees,
    events,
    gift_cards,
    inventory,
    invoices,
    labor,
    locations,
    loyalty,
    merchants,
    mobile,
    o_auth,
    orders,
    payments,
    payouts,
    refunds,
    sites,
    snippets,
    subscriptions,
    team,
    team_members,
    terminal,
    v1transactions,
    vendors,
    webhooks,
)
from .client import AsyncSquare, Square
from .version import __version__

__all__ = [
    "AsyncSquare",
    "Square",
    "__version__",
    "apple_pay",
    "bank_accounts",
    "bookings",
    "cards",
    "cash_drawers",
    "catalog",
    "checkout",
    "customers",
    "devices",
    "disputes",
    "employees",
    "events",
    "gift_cards",
    "inventory",
    "invoices",
    "labor",
    "locations",
    "loyalty",
    "merchants",
    "mobile",
    "o_auth",
    "orders",
    "payments",
    "payouts",
    "refunds",
    "sites",
    "snippets",
    "subscriptions",
    "team",
    "team_members",
    "terminal",
    "v1transactions",
    "vendors",
    "webhooks",
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .delete_customer_response import DeleteCustomerResponseParams
from .error import ErrorParams


class BulkDeleteCustomersResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields included in the response body from the
    [BulkDeleteCustomers](api-endpoint:Customers-BulkDeleteCustomers) endpoint.
    """

    responses: typing_extensions.NotRequired[typing.Dict[str, DeleteCustomerResponseParams]]
    """
    A map of responses that correspond to individual delete requests, represented by
    key-value pairs.
    
    Each key is the customer ID that was specified for a delete request and each value
    is the corresponding response.
    If the request succeeds, the value is an empty object (`{ }`).
    If the request fails, the value contains any errors that occurred during the request.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any top-level errors that prevented the bulk operation from running.
    """



# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawTransactionsClient
import typing
from ...types.sort_order import SortOrder
from ...core.request_options import RequestOptions
from ...types.list_transactions_response import ListTransactionsResponse
from ...types.get_transaction_response import GetTransactionResponse
from ...types.capture_transaction_response import CaptureTransactionResponse
from ...types.void_transaction_response import VoidTransactionResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawTransactionsClient


class TransactionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTransactionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTransactionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTransactionsClient
        """
        return self._raw_client

    def list(
        self,
        location_id: str,
        *,
        begin_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        sort_order: typing.Optional[SortOrder] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListTransactionsResponse:
        """
        Lists transactions for a particular location.

        Transactions include payment information from sales and exchanges and refund
        information from returns and exchanges.

        Max results per [page](https://developer.squareup.com/docs/working-with-apis/pagination): 50

        Parameters
        ----------
        location_id : str
            The ID of the location to list transactions for.

        begin_time : typing.Optional[str]
            The beginning of the requested reporting period, in RFC 3339 format.

            See [Date ranges](https://developer.squareup.com/docs/build-basics/working-with-dates) for details on date inclusivity/exclusivity.

            Default value: The current time minus one year.

        end_time : typing.Optional[str]
            The end of the requested reporting period, in RFC 3339 format.

            See [Date ranges](https://developer.squareup.com/docs/build-basics/working-with-dates) for details on date inclusivity/exclusivity.

            Default value: The current time.

        sort_order : typing.Optional[SortOrder]
            The order in which results are listed in the response (`ASC` for
            oldest first, `DESC` for newest first).

            Default value: `DESC`

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for your original query.

            See [Paginating results](https://developer.squareup.com/docs/working-with-apis/pagination) for more information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListTransactionsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.transactions.list(
            location_id="location_id",
        )
        """
        response = self._raw_client.list(
            location_id,
            begin_time=begin_time,
            end_time=end_time,
            sort_order=sort_order,
            cursor=cursor,
            request_options=request_options,
        )
        return response.data

    def get(
        self, location_id: str, transaction_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTransactionResponse:
        """
        Retrieves details for a single transaction.

        Parameters
        ----------
        location_id : str
            The ID of the transaction's associated location.

        transaction_id : str
            The ID of the transaction to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTransactionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.transactions.get(
            location_id="location_id",
            transaction_id="transaction_id",
        )
        """
        response = self._raw_client.get(location_id, transaction_id, request_options=request_options)
        return response.data

    def capture(
        self, location_id: str, transaction_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CaptureTransactionResponse:
        """
        Captures a transaction that was created with the [Charge](api-endpoint:Transactions-Charge)
        endpoint with a `delay_capture` value of `true`.


        See [Delayed capture transactions](https://developer.squareup.com/docs/payments/transactions/overview#delayed-capture)
        for more information.

        Parameters
        ----------
        location_id : str


        transaction_id : str


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CaptureTransactionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.transactions.capture(
            location_id="location_id",
            transaction_id="transaction_id",
        )
        """
        response = self._raw_client.capture(location_id, transaction_id, request_options=request_options)
        return response.data

    def void(
        self, location_id: str, transaction_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> VoidTransactionResponse:
        """
        Cancels a transaction that was created with the [Charge](api-endpoint:Transactions-Charge)
        endpoint with a `delay_capture` value of `true`.


        See [Delayed capture transactions](https://developer.squareup.com/docs/payments/transactions/overview#delayed-capture)
        for more information.

        Parameters
        ----------
        location_id : str


        transaction_id : str


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VoidTransactionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.transactions.void(
            location_id="location_id",
            transaction_id="transaction_id",
        )
        """
        response = self._raw_client.void(location_id, transaction_id, request_options=request_options)
        return response.data


class AsyncTransactionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTransactionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTransactionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTransactionsClient
        """
        return self._raw_client

    async def list(
        self,
        location_id: str,
        *,
        begin_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        sort_order: typing.Optional[SortOrder] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListTransactionsResponse:
        """
        Lists transactions for a particular location.

        Transactions include payment information from sales and exchanges and refund
        information from returns and exchanges.

        Max results per [page](https://developer.squareup.com/docs/working-with-apis/pagination): 50

        Parameters
        ----------
        location_id : str
            The ID of the location to list transactions for.

        begin_time : typing.Optional[str]
            The beginning of the requested reporting period, in RFC 3339 format.

            See [Date ranges](https://developer.squareup.com/docs/build-basics/working-with-dates) for details on date inclusivity/exclusivity.

            Default value: The current time minus one year.

        end_time : typing.Optional[str]
            The end of the requested reporting period, in RFC 3339 format.

            See [Date ranges](https://developer.squareup.com/docs/build-basics/working-with-dates) for details on date inclusivity/exclusivity.

            Default value: The current time.

        sort_order : typing.Optional[SortOrder]
            The order in which results are listed in the response (`ASC` for
            oldest first, `DESC` for newest first).

            Default value: `DESC`

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for your original query.

            See [Paginating results](https://developer.squareup.com/docs/working-with-apis/pagination) for more information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListTransactionsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.transactions.list(
                location_id="location_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.list(
            location_id,
            begin_time=begin_time,
            end_time=end_time,
            sort_order=sort_order,
            cursor=cursor,
            request_options=request_options,
        )
        return response.data

    async def get(
        self, location_id: str, transaction_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTransactionResponse:
        """
        Retrieves details for a single transaction.

        Parameters
        ----------
        location_id : str
            The ID of the transaction's associated location.

        transaction_id : str
            The ID of the transaction to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTransactionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.transactions.get(
                location_id="location_id",
                transaction_id="transaction_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(location_id, transaction_id, request_options=request_options)
        return response.data

    async def capture(
        self, location_id: str, transaction_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CaptureTransactionResponse:
        """
        Captures a transaction that was created with the [Charge](api-endpoint:Transactions-Charge)
        endpoint with a `delay_capture` value of `true`.


        See [Delayed capture transactions](https://developer.squareup.com/docs/payments/transactions/overview#delayed-capture)
        for more information.

        Parameters
        ----------
        location_id : str


        transaction_id : str


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CaptureTransactionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.transactions.capture(
                location_id="location_id",
                transaction_id="transaction_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.capture(location_id, transaction_id, request_options=request_options)
        return response.data

    async def void(
        self, location_id: str, transaction_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> VoidTransactionResponse:
        """
        Cancels a transaction that was created with the [Charge](api-endpoint:Transactions-Charge)
        endpoint with a `delay_capture` value of `true`.


        See [Delayed capture transactions](https://developer.squareup.com/docs/payments/transactions/overview#delayed-capture)
        for more information.

        Parameters
        ----------
        location_id : str


        transaction_id : str


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VoidTransactionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.transactions.void(
                location_id="location_id",
                transaction_id="transaction_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.void(location_id, transaction_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawVendorsClient
from ..requests.vendor import VendorParams
from ..core.request_options import RequestOptions
from ..types.batch_create_vendors_response import BatchCreateVendorsResponse
from ..types.batch_get_vendors_response import BatchGetVendorsResponse
from ..requests.update_vendor_request import UpdateVendorRequestParams
from ..types.batch_update_vendors_response import BatchUpdateVendorsResponse
from ..types.create_vendor_response import CreateVendorResponse
from ..requests.search_vendors_request_filter import SearchVendorsRequestFilterParams
from ..requests.search_vendors_request_sort import SearchVendorsRequestSortParams
from ..types.search_vendors_response import SearchVendorsResponse
from ..types.get_vendor_response import GetVendorResponse
from ..types.update_vendor_response import UpdateVendorResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawVendorsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class VendorsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawVendorsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawVendorsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawVendorsClient
        """
        return self._raw_client

    def batch_create(
        self, *, vendors: typing.Dict[str, VendorParams], request_options: typing.Optional[RequestOptions] = None
    ) -> BatchCreateVendorsResponse:
        """
        Creates one or more [Vendor](entity:Vendor) objects to represent suppliers to a seller.

        Parameters
        ----------
        vendors : typing.Dict[str, VendorParams]
            Specifies a set of new [Vendor](entity:Vendor) objects as represented by a collection of idempotency-key/`Vendor`-object pairs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchCreateVendorsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.vendors.batch_create(
            vendors={
                "8fc6a5b0-9fe8-4b46-b46b-2ef95793abbe": {
                    "name": "Joe's Fresh Seafood",
                    "address": {
                        "address_line1": "505 Electric Ave",
                        "address_line2": "Suite 600",
                        "locality": "New York",
                        "administrative_district_level1": "NY",
                        "postal_code": "10003",
                        "country": "US",
                    },
                    "contacts": [
                        {
                            "name": "Joe Burrow",
                            "email_address": "joe@joesfreshseafood.com",
                            "phone_number": "1-212-555-4250",
                            "ordinal": 1,
                        }
                    ],
                    "account_number": "4025391",
                    "note": "a vendor",
                }
            },
        )
        """
        response = self._raw_client.batch_create(vendors=vendors, request_options=request_options)
        return response.data

    def batch_get(
        self,
        *,
        vendor_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchGetVendorsResponse:
        """
        Retrieves one or more vendors of specified [Vendor](entity:Vendor) IDs.

        Parameters
        ----------
        vendor_ids : typing.Optional[typing.Sequence[str]]
            IDs of the [Vendor](entity:Vendor) objects to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchGetVendorsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.vendors.batch_get(
            vendor_ids=["INV_V_JDKYHBWT1D4F8MFH63DBMEN8Y4"],
        )
        """
        response = self._raw_client.batch_get(vendor_ids=vendor_ids, request_options=request_options)
        return response.data

    def batch_update(
        self,
        *,
        vendors: typing.Dict[str, UpdateVendorRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchUpdateVendorsResponse:
        """
        Updates one or more of existing [Vendor](entity:Vendor) objects as suppliers to a seller.

        Parameters
        ----------
        vendors : typing.Dict[str, UpdateVendorRequestParams]
            A set of [UpdateVendorRequest](entity:UpdateVendorRequest) objects encapsulating to-be-updated [Vendor](entity:Vendor)
            objects. The set is represented by  a collection of `Vendor`-ID/`UpdateVendorRequest`-object pairs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchUpdateVendorsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.vendors.batch_update(
            vendors={
                "FMCYHBWT1TPL8MFH52PBMEN92A": {"vendor": {}},
                "INV_V_JDKYHBWT1D4F8MFH63DBMEN8Y4": {"vendor": {}},
            },
        )
        """
        response = self._raw_client.batch_update(vendors=vendors, request_options=request_options)
        return response.data

    def create(
        self,
        *,
        idempotency_key: str,
        vendor: typing.Optional[VendorParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateVendorResponse:
        """
        Creates a single [Vendor](entity:Vendor) object to represent a supplier to a seller.

        Parameters
        ----------
        idempotency_key : str
            A client-supplied, universally unique identifier (UUID) to make this [CreateVendor](api-endpoint:Vendors-CreateVendor) call idempotent.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        vendor : typing.Optional[VendorParams]
            The requested [Vendor](entity:Vendor) to be created.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateVendorResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.vendors.create(
            idempotency_key="8fc6a5b0-9fe8-4b46-b46b-2ef95793abbe",
            vendor={
                "name": "Joe's Fresh Seafood",
                "address": {
                    "address_line1": "505 Electric Ave",
                    "address_line2": "Suite 600",
                    "locality": "New York",
                    "administrative_district_level1": "NY",
                    "postal_code": "10003",
                    "country": "US",
                },
                "contacts": [
                    {
                        "name": "Joe Burrow",
                        "email_address": "joe@joesfreshseafood.com",
                        "phone_number": "1-212-555-4250",
                        "ordinal": 1,
                    }
                ],
                "account_number": "4025391",
                "note": "a vendor",
            },
        )
        """
        response = self._raw_client.create(
            idempotency_key=idempotency_key, vendor=vendor, request_options=request_options
        )
        return response.data

    def search(
        self,
        *,
        filter: typing.Optional[SearchVendorsRequestFilterParams] = OMIT,
        sort: typing.Optional[SearchVendorsRequestSortParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchVendorsResponse:
        """
        Searches for vendors using a filter against supported [Vendor](entity:Vendor) properties and a supported sorter.

        Parameters
        ----------
        filter : typing.Optional[SearchVendorsRequestFilterParams]
            Specifies a filter used to search for vendors.

        sort : typing.Optional[SearchVendorsRequestSortParams]
            Specifies a sorter used to sort the returned vendors.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchVendorsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.vendors.search()
        """
        response = self._raw_client.search(filter=filter, sort=sort, cursor=cursor, request_options=request_options)
        return response.data

    def get(self, vendor_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetVendorResponse:
        """
        Retrieves the vendor of a specified [Vendor](entity:Vendor) ID.

        Parameters
        ----------
        vendor_id : str
            ID of the [Vendor](entity:Vendor) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetVendorResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.vendors.get(
            vendor_id="vendor_id",
        )
        """
        response = self._raw_client.get(vendor_id, request_options=request_options)
        return response.data

    def update(
        self,
        vendor_id: str,
        *,
        vendor: VendorParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateVendorResponse:
        """
        Updates an existing [Vendor](entity:Vendor) object as a supplier to a seller.

        Parameters
        ----------
        vendor_id : str
            ID of the [Vendor](entity:Vendor) to retrieve.

        vendor : VendorParams
            The specified [Vendor](entity:Vendor) to be updated.

        idempotency_key : typing.Optional[str]
            A client-supplied, universally unique identifier (UUID) for the
            request.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateVendorResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.vendors.update(
            vendor_id="vendor_id",
            idempotency_key="8fc6a5b0-9fe8-4b46-b46b-2ef95793abbe",
            vendor={
                "id": "INV_V_JDKYHBWT1D4F8MFH63DBMEN8Y4",
                "name": "Jack's Chicken Shack",
                "version": 1,
                "status": "ACTIVE",
            },
        )
        """
        response = self._raw_client.update(
            vendor_id, vendor=vendor, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data


class AsyncVendorsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawVendorsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawVendorsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawVendorsClient
        """
        return self._raw_client

    async def batch_create(
        self, *, vendors: typing.Dict[str, VendorParams], request_options: typing.Optional[RequestOptions] = None
    ) -> BatchCreateVendorsResponse:
        """
        Creates one or more [Vendor](entity:Vendor) objects to represent suppliers to a seller.

        Parameters
        ----------
        vendors : typing.Dict[str, VendorParams]
            Specifies a set of new [Vendor](entity:Vendor) objects as represented by a collection of idempotency-key/`Vendor`-object pairs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchCreateVendorsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.vendors.batch_create(
                vendors={
                    "8fc6a5b0-9fe8-4b46-b46b-2ef95793abbe": {
                        "name": "Joe's Fresh Seafood",
                        "address": {
                            "address_line1": "505 Electric Ave",
                            "address_line2": "Suite 600",
                            "locality": "New York",
                            "administrative_district_level1": "NY",
                            "postal_code": "10003",
                            "country": "US",
                        },
                        "contacts": [
                            {
                                "name": "Joe Burrow",
                                "email_address": "joe@joesfreshseafood.com",
                                "phone_number": "1-212-555-4250",
                                "ordinal": 1,
                            }
                        ],
                        "account_number": "4025391",
                        "note": "a vendor",
                    }
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.batch_create(vendors=vendors, request_options=request_options)
        return response.data

    async def batch_get(
        self,
        *,
        vendor_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchGetVendorsResponse:
        """
        Retrieves one or more vendors of specified [Vendor](entity:Vendor) IDs.

        Parameters
        ----------
        vendor_ids : typing.Optional[typing.Sequence[str]]
            IDs of the [Vendor](entity:Vendor) objects to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchGetVendorsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.vendors.batch_get(
                vendor_ids=["INV_V_JDKYHBWT1D4F8MFH63DBMEN8Y4"],
            )


        asyncio.run(main())
        """
        response = await self._raw_client.batch_get(vendor_ids=vendor_ids, request_options=request_options)
        return response.data

    async def batch_update(
        self,
        *,
        vendors: typing.Dict[str, UpdateVendorRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchUpdateVendorsResponse:
        """
        Updates one or more of existing [Vendor](entity:Vendor) objects as suppliers to a seller.

        Parameters
        ----------
        vendors : typing.Dict[str, UpdateVendorRequestParams]
            A set of [UpdateVendorRequest](entity:UpdateVendorRequest) objects encapsulating to-be-updated [Vendor](entity:Vendor)
            objects. The set is represented by  a collection of `Vendor`-ID/`UpdateVendorRequest`-object pairs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchUpdateVendorsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.vendors.batch_update(
                vendors={
                    "FMCYHBWT1TPL8MFH52PBMEN92A": {"vendor": {}},
                    "INV_V_JDKYHBWT1D4F8MFH63DBMEN8Y4": {"vendor": {}},
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.batch_update(vendors=vendors, request_options=request_options)
        return response.data

    async def create(
        self,
        *,
        idempotency_key: str,
        vendor: typing.Optional[VendorParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateVendorResponse:
        """
        Creates a single [Vendor](entity:Vendor) object to represent a supplier to a seller.

        Parameters
        ----------
        idempotency_key : str
            A client-supplied, universally unique identifier (UUID) to make this [CreateVendor](api-endpoint:Vendors-CreateVendor) call idempotent.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        vendor : typing.Optional[VendorParams]
            The requested [Vendor](entity:Vendor) to be created.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateVendorResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.vendors.create(
                idempotency_key="8fc6a5b0-9fe8-4b46-b46b-2ef95793abbe",
                vendor={
                    "name": "Joe's Fresh Seafood",
                    "address": {
                        "address_line1": "505 Electric Ave",
                        "address_line2": "Suite 600",
                        "locality": "New York",
                        "administrative_district_level1": "NY",
                        "postal_code": "10003",
                        "country": "US",
                    },
                    "contacts": [
                        {
                            "name": "Joe Burrow",
                            "email_address": "joe@joesfreshseafood.com",
                            "phone_number": "1-212-555-4250",
                            "ordinal": 1,
                        }
                    ],
                    "account_number": "4025391",
                    "note": "a vendor",
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            idempotency_key=idempotency_key, vendor=vendor, request_options=request_options
        )
        return response.data

    async def search(
        self,
        *,
        filter: typing.Optional[SearchVendorsRequestFilterParams] = OMIT,
        sort: typing.Optional[SearchVendorsRequestSortParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchVendorsResponse:
        """
        Searches for vendors using a filter against supported [Vendor](entity:Vendor) properties and a supported sorter.

        Parameters
        ----------
        filter : typing.Optional[SearchVendorsRequestFilterParams]
            Specifies a filter used to search for vendors.

        sort : typing.Optional[SearchVendorsRequestSortParams]
            Specifies a sorter used to sort the returned vendors.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchVendorsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.vendors.search()


        asyncio.run(main())
        """
        response = await self._raw_client.search(
            filter=filter, sort=sort, cursor=cursor, request_options=request_options
        )
        return response.data

    async def get(
        self, vendor_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetVendorResponse:
        """
        Retrieves the vendor of a specified [Vendor](entity:Vendor) ID.

        Parameters
        ----------
        vendor_id : str
            ID of the [Vendor](entity:Vendor) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetVendorResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.vendors.get(
                vendor_id="vendor_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(vendor_id, request_options=request_options)
        return response.data

    async def update(
        self,
        vendor_id: str,
        *,
        vendor: VendorParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateVendorResponse:
        """
        Updates an existing [Vendor](entity:Vendor) object as a supplier to a seller.

        Parameters
        ----------
        vendor_id : str
            ID of the [Vendor](entity:Vendor) to retrieve.

        vendor : VendorParams
            The specified [Vendor](entity:Vendor) to be updated.

        idempotency_key : typing.Optional[str]
            A client-supplied, universally unique identifier (UUID) for the
            request.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateVendorResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.vendors.update(
                vendor_id="vendor_id",
                idempotency_key="8fc6a5b0-9fe8-4b46-b46b-2ef95793abbe",
                vendor={
                    "id": "INV_V_JDKYHBWT1D4F8MFH63DBMEN8Y4",
                    "name": "Jack's Chicken Shack",
                    "version": 1,
                    "status": "ACTIVE",
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(
            vendor_id, vendor=vendor, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .order_state import OrderState
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchOrdersStateFilter(UncheckedBaseModel):
    """
    Filter by the current order `state`.
    """

    states: typing.List[OrderState] = pydantic.Field()
    """
    States to filter for.
    See [OrderState](#type-orderstate) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .workweek_config import WorkweekConfigParams
import typing
from .error import ErrorParams


class UpdateWorkweekConfigResponseParams(typing_extensions.TypedDict):
    """
    The response to a request to update a `WorkweekConfig` object. The response contains
    the updated `WorkweekConfig` object and might contain a set of `Error` objects if
    the request resulted in errors.
    """

    workweek_config: typing_extensions.NotRequired[WorkweekConfigParams]
    """
    The response object.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .money import MoneyParams


class CheckoutMerchantSettingsPaymentMethodsAfterpayClearpayEligibilityRangeParams(typing_extensions.TypedDict):
    """
    A range of purchase price that qualifies.
    """

    min: MoneyParams
    max: MoneyParams



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawMobileClient
from ..core.request_options import RequestOptions
from ..types.create_mobile_authorization_code_response import CreateMobileAuthorizationCodeResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawMobileClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class MobileClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMobileClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawMobileClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMobileClient
        """
        return self._raw_client

    def authorization_code(
        self, *, location_id: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateMobileAuthorizationCodeResponse:
        """
        __Note:__ This endpoint is used by the deprecated Reader SDK.
        Developers should update their integration to use the [Mobile Payments SDK](https://developer.squareup.com/docs/mobile-payments-sdk), which includes its own authorization methods.

        Generates code to authorize a mobile application to connect to a Square card reader.

        Authorization codes are one-time-use codes and expire 60 minutes after being issued.

        The `Authorization` header you provide to this endpoint must have the following format:

        ```
        Authorization: Bearer ACCESS_TOKEN
        ```

        Replace `ACCESS_TOKEN` with a
        [valid production authorization credential](https://developer.squareup.com/docs/build-basics/access-tokens).

        Parameters
        ----------
        location_id : typing.Optional[str]
            The Square location ID that the authorization code should be tied to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateMobileAuthorizationCodeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.mobile.authorization_code(
            location_id="YOUR_LOCATION_ID",
        )
        """
        response = self._raw_client.authorization_code(location_id=location_id, request_options=request_options)
        return response.data


class AsyncMobileClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMobileClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawMobileClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMobileClient
        """
        return self._raw_client

    async def authorization_code(
        self, *, location_id: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateMobileAuthorizationCodeResponse:
        """
        __Note:__ This endpoint is used by the deprecated Reader SDK.
        Developers should update their integration to use the [Mobile Payments SDK](https://developer.squareup.com/docs/mobile-payments-sdk), which includes its own authorization methods.

        Generates code to authorize a mobile application to connect to a Square card reader.

        Authorization codes are one-time-use codes and expire 60 minutes after being issued.

        The `Authorization` header you provide to this endpoint must have the following format:

        ```
        Authorization: Bearer ACCESS_TOKEN
        ```

        Replace `ACCESS_TOKEN` with a
        [valid production authorization credential](https://developer.squareup.com/docs/build-basics/access-tokens).

        Parameters
        ----------
        location_id : typing.Optional[str]
            The Square location ID that the authorization code should be tied to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateMobileAuthorizationCodeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.mobile.authorization_code(
                location_id="YOUR_LOCATION_ID",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.authorization_code(location_id=location_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .fulfillment_recipient import FulfillmentRecipientParams
import typing
from ..types.fulfillment_pickup_details_schedule_type import FulfillmentPickupDetailsScheduleType
from .fulfillment_pickup_details_curbside_pickup_details import FulfillmentPickupDetailsCurbsidePickupDetailsParams


class FulfillmentPickupDetailsParams(typing_extensions.TypedDict):
    """
    Contains details necessary to fulfill a pickup order.
    """

    recipient: typing_extensions.NotRequired[FulfillmentRecipientParams]
    """
    Information about the person to pick up this fulfillment from a physical
    location.
    """

    expires_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when this fulfillment expires if it is not marked in progress. The timestamp must be
    in RFC 3339 format (for example, "2016-09-04T23:59:33.123Z"). The expiration time can only be set
    up to 7 days in the future. If `expires_at` is not set, any new payments attached to the order
    are automatically completed.
    """

    auto_complete_duration: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The duration of time after which an in progress pickup fulfillment is automatically moved
    to the `COMPLETED` state. The duration must be in RFC 3339 format (for example, "P1W3D").
    
    If not set, this pickup fulfillment remains in progress until it is canceled or completed.
    """

    schedule_type: typing_extensions.NotRequired[FulfillmentPickupDetailsScheduleType]
    """
    The schedule type of the pickup fulfillment. Defaults to `SCHEDULED`.
    See [FulfillmentPickupDetailsScheduleType](#type-fulfillmentpickupdetailsscheduletype) for possible values
    """

    pickup_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    that represents the start of the pickup window. Must be in RFC 3339 timestamp format, e.g.,
    "2016-09-04T23:59:33.123Z".
    
    For fulfillments with the schedule type `ASAP`, this is automatically set
    to the current time plus the expected duration to prepare the fulfillment.
    """

    pickup_window_duration: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The window of time in which the order should be picked up after the `pickup_at` timestamp.
    Must be in RFC 3339 duration format, e.g., "P1W3D". Can be used as an
    informational guideline for merchants.
    """

    prep_time_duration: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The duration of time it takes to prepare this fulfillment.
    The duration must be in RFC 3339 format (for example, "P1W3D").
    """

    note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A note to provide additional instructions about the pickup
    fulfillment displayed in the Square Point of Sale application and set by the API.
    """

    placed_at: typing_extensions.NotRequired[str]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the fulfillment was placed. The timestamp must be in RFC 3339 format
    (for example, "2016-09-04T23:59:33.123Z").
    """

    accepted_at: typing_extensions.NotRequired[str]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the fulfillment was marked in progress. The timestamp must be in RFC 3339 format
    (for example, "2016-09-04T23:59:33.123Z").
    """

    rejected_at: typing_extensions.NotRequired[str]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the fulfillment was rejected. The timestamp must be in RFC 3339 format
    (for example, "2016-09-04T23:59:33.123Z").
    """

    ready_at: typing_extensions.NotRequired[str]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the fulfillment is marked as ready for pickup. The timestamp must be in RFC 3339 format
    (for example, "2016-09-04T23:59:33.123Z").
    """

    expired_at: typing_extensions.NotRequired[str]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the fulfillment expired. The timestamp must be in RFC 3339 format
    (for example, "2016-09-04T23:59:33.123Z").
    """

    picked_up_at: typing_extensions.NotRequired[str]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the fulfillment was picked up by the recipient. The timestamp must be in RFC 3339 format
    (for example, "2016-09-04T23:59:33.123Z").
    """

    canceled_at: typing_extensions.NotRequired[str]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the fulfillment was canceled. The timestamp must be in RFC 3339 format
    (for example, "2016-09-04T23:59:33.123Z").
    """

    cancel_reason: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A description of why the pickup was canceled. The maximum length: 100 characters.
    """

    is_curbside_pickup: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    If set to `true`, indicates that this pickup order is for curbside pickup, not in-store pickup.
    """

    curbside_pickup_details: typing_extensions.NotRequired[FulfillmentPickupDetailsCurbsidePickupDetailsParams]
    """
    Specific details for curbside pickup. These details can only be populated if `is_curbside_pickup` is set to `true`.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .terminal_refund import TerminalRefund
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchTerminalRefundsResponse(UncheckedBaseModel):
    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Information about errors encountered during the request.
    """

    refunds: typing.Optional[typing.List[TerminalRefund]] = pydantic.Field(default=None)
    """
    The requested search result of `TerminalRefund` objects.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    The pagination cursor to be used in a subsequent request. If empty,
    this is the final response.
    
    See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more information.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .currency import Currency
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Money(UncheckedBaseModel):
    """
    Represents an amount of money. `Money` fields can be signed or unsigned.
    Fields that do not explicitly define whether they are signed or unsigned are
    considered unsigned and can only hold positive amounts. For signed fields, the
    sign of the value indicates the purpose of the money transfer. See
    [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts)
    for more information.
    """

    amount: typing.Optional[int] = pydantic.Field(default=None)
    """
    The amount of money, in the smallest denomination of the currency
    indicated by `currency`. For example, when `currency` is `USD`, `amount` is
    in cents. Monetary amounts can be positive or negative. See the specific
    field description to determine the meaning of the sign in a particular case.
    """

    currency: typing.Optional[Currency] = pydantic.Field(default=None)
    """
    The type of currency, in __ISO 4217 format__. For example, the currency
    code for US dollars is `USD`.
    
    See [Currency](entity:Currency) for possible values.
    See [Currency](#type-currency) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .payment_link import PaymentLink
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ListPaymentLinksResponse(UncheckedBaseModel):
    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Errors that occurred during the request.
    """

    payment_links: typing.Optional[typing.List[PaymentLink]] = pydantic.Field(default=None)
    """
    The list of payment links.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
      When a response is truncated, it includes a cursor that you can use in a subsequent request
    to retrieve the next set of gift cards. If a cursor is not present, this is the final response.
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
